package OBM::Update::updateBdQuotaUse;

$VERSION = "1.0";

$debug = 1;

use 5.006_001;
require Exporter;
use strict;

require OBM::toolBox;
require OBM::dbUtils;
require OBM::imapd;
require OBM::Update::utils;
require OBM::Cyrus::cyrusEngine;


sub new {
    my $self = shift;
    my( $dbHandler, $parameters ) = @_;

    my %updateBdQuotaUse = (
        dbHandler => undef,
        domainId => undef,
        domainList => undef,
        engine => undef
    );

    if( !defined($dbHandler) || !defined($parameters) ) {
        croak( "[Update::updateBdQuotaUse]: Usage: PACKAGE->new(DBHANDLER, PARAMLIST)" );
    }

    $updateBdQuotaUse{dbHandler} = $dbHandler;

    if( !exists($parameters->{'all'}) ) {
        $updateBdQuotaUse{'domainId'} = $parameters->{'domain'};
        if( $updateBdQuotaUse{'domainId'} == 0 ) {
            &OBM::toolBox::write_log( '[Update::updateBdQuotaUse]: pas de support du domaine d\'ID \'0\'', 'W' );
            return undef;
        }
    }

    # Obtention des informations sur les domaines nécessaires
    $updateBdQuotaUse{'domainList'} = &OBM::Update::utils::getDomains( $updateBdQuotaUse{'dbHandler'}, $updateBdQuotaUse{'domainId'} );

    # Paramétrage des serveurs IMAP par domaine
    &OBM::Update::utils::getCyrusServers( $updateBdQuotaUse{"dbHandler"}, $updateBdQuotaUse{"domainList"} );
    if( !&OBM::imapd::getAdminImapPasswd( $updateBdQuotaUse{"dbHandler"}, $updateBdQuotaUse{"domainList"} ) ) {
        return undef;
    }

    # Initialisation du moteur Cyrus
    $updateBdQuotaUse{"engine"}->{"cyrusEngine"} = OBM::Cyrus::cyrusEngine->new( $updateBdQuotaUse{"domainList"} );
    if( !$updateBdQuotaUse{"engine"}->{"cyrusEngine"}->init() ) {
        return undef;
    }


    bless( \%updateBdQuotaUse, $self );
}


sub update {
    my $self = shift;
    my $domainId = $self->{'domainId'};

    if( defined($domainId) ) {
        $self->updateUserQuota( $domainId );
        $self->updateMailshareQuota( $domainId );
    }else {
        my $dbHandler = $self->{'dbHandler'};
        my $queryResult;

        my $query = 'SELECT domain_id FROM Domain WHERE domain_id!=0';
        if( !&OBM::dbUtils::execQuery( $query, $dbHandler, \$queryResult ) ) {
            &OBM::toolBox::write_log( '[Update::updateBdQuotaUse]: probleme lors de l\'execution d\'une requete SQL : ', 'W' );
            return 1;
        }

        while( ( $domainId ) = $queryResult->fetchrow_array() ) {
            $self->updateUserQuota( $domainId );
            $self->updateMailshareQuota( $domainId );
        }
    }
    
    return 0;
}


sub updateUserQuota {
    my $self = shift;
    my( $domainId ) = @_;
    my $cyrusEngine = $self->{"engine"}->{"cyrusEngine"};
    my $domainList = $self->{'domainList'};
    my $dbHandler = $self->{'dbHandler'};

    my $query = 'SELECT userobm_id FROM UserObm
                    WHERE userobm_domain_id='.$domainId.'
                    AND userobm_mail_perms=1';
    my $queryResult;
    if( !&OBM::dbUtils::execQuery( $query, $dbHandler, \$queryResult ) ) {
        &OBM::toolBox::write_log( '[Update::updateBdQuotaUse]: probleme lors de l\'execution d\'une requete SQL : ', 'W' );
        return 1;
    }


    require OBM::Entities::obmUser;
    while( my( $userId ) = $queryResult->fetchrow_array() ) {
        my $object = OBM::Entities::obmUser->new( 1, 0, $userId );
        $object->getEntity( $dbHandler, &OBM::Update::utils::findDomainbyId( $domainList, $domainId ) );

        &OBM::toolBox::write_log( '[Update::updateBdQuotaUse]: mise a jour du quota mail utilise de l\'entite '.$object->getEntityDescription(), 'W' );

        my $quotaUsed = $cyrusEngine->getMailboxQuotaUse( $object );
        
        $query = 'UPDATE UserObm
                    SET userobm_mail_quota_use='.int($quotaUsed).'
                    WHERE userobm_id='.$userId;
        my $queryUpdateResult;
        if( !&OBM::dbUtils::execQuery( $query, $dbHandler, \$queryUpdateResult ) ) {
            &OBM::toolBox::write_log( '[Update::updateBdQuotaUse]: probleme lors de l\'execution d\'une requete SQL : ', 'W' );
            next;
        }

        $query = 'UPDATE P_UserObm
                    SET userobm_mail_quota_use='.int($quotaUsed).'
                    WHERE userobm_id='.$userId;
        if( !&OBM::dbUtils::execQuery( $query, $dbHandler, \$queryUpdateResult ) ) {
            &OBM::toolBox::write_log( '[Update::updateBdQuotaUse]: probleme lors de l\'execution d\'une requete SQL : ', 'W' );
            next;
        }
    }

    return 0;
}


sub updateMailshareQuota {
    my $self = shift;
    my( $domainId ) = @_;
    my $cyrusEngine = $self->{"engine"}->{"cyrusEngine"};
    my $dbHandler = $self->{'dbHandler'};

    return 0;
}


sub destroy {
    my $self = shift;
    my $cyrusEngine = $self->{"engine"}->{"cyrusEngine"};

    $cyrusEngine->destroy();
}
