#!/usr/bin/perl -w -T

#+-------------------------------------------------------------------------+
#|   Copyright (c) 1997-2009 OBM.org project members team                  |
#|                                                                         |
#|  This program is free software; you can redistribute it and/or          |
#|  modify it under the terms of the GNU General Public License            |
#|  as published by the Free Software Foundation; version 2                |
#|  of the License.                                                        |
#|                                                                         |
#|  This program is distributed in the hope that it will be useful,        |
#|  but WITHOUT ANY WARRANTY; without even the implied warranty of         |
#|  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
#|  GNU General Public License for more details.                           | 
#+-------------------------------------------------------------------------+
#|  http://www.obm.org                                                     |
#+-------------------------------------------------------------------------+

#####################################################################
# OBM               - File : updateBdQuotaUse.pl                    #
#                   - Desc : script permettant une mise à jour en   #
#                            temps réel des ACLs sur une BAL/share  #
#####################################################################

use strict;
require OBM::toolBox;
require OBM::Update::updateBdQuotaUse;
use OBM::Parameters::common;
use Getopt::Long;


delete @ENV{qw(IFS CDPATH ENV BASH_ENV PATH)};


# Fonction de vérification des paramètres du script
sub getParameter {
    my( $parameters ) = @_;

    # Analyse de la ligne de commande
    &GetOptions( $parameters, "domain=s", "all" );

    SWITCH: {
        if( exists($$parameters{'all'}) ) {
            &OBM::toolBox::write_log( 'Mise a jour de l\'ensemble des domaines OBM', 'W' );
            delete($$parameters{'domain'});
            last SWITCH;
        }

        # Vérification du type d'entité à mettre à jour
        if( !exists($$parameters{"domain"}) ) {
            &OBM::toolBox::write_log( "Parametre --domain manquant", "W" );
            print STDERR "Paramètre '--domain' manquant.\nCe paramètre indique l'ID BD du domaine de l'entité à mettre à jour.\nParamètre '--all' pour mettre à jour l'ensemble des domaines OBM\n";
            return 1;
        }else {
            if( $$parameters{"domain"} !~ /^[0-9]+$/ ) {
                &OBM::toolBox::write_log( "Parametre --domain invalide", "W" );
                print STDERR "Paramètre '--domain' invalide.\nCe paramètre indique l'ID BD du domaine de l'entité à mettre à jour.\nParamètre '--all' pour mettre à jour l'ensemble des domaines OBM\n";
                return 1;
            }
        }
    }

    return 0;
}

# On prepare le log
&OBM::toolBox::write_log( 'updateBdQuotaUse.pl: ', 'O' );

# Traitement des paramètres
&OBM::toolBox::write_log( 'Analyse des parametres du script', 'W' );
my %parameters;
if( getParameter( \%parameters ) ) {
    &OBM::toolBox::write_log( '', 'C' );
    exit 1;
}


# On se connecte à la base
my $dbHandler;
&OBM::toolBox::write_log( 'Connexion a la base de donnees OBM', 'W' );
if( !&OBM::dbUtils::dbState( 'connect', \$dbHandler ) ) {
    &OBM::toolBox::write_log( 'Probleme lors de l\'ouverture de la base de donnees : '.$dbHandler->err, 'WC' );
    exit 1;
}


my $errorCode = 0;
my $updateBdQuotaUse = OBM::Update::updateBdQuotaUse->new( $dbHandler, \%parameters );
if( defined($updateBdQuotaUse) ) {
    $errorCode = $updateBdQuotaUse->update();
    $updateBdQuotaUse->destroy();
}


# On referme la connexion à la base
&OBM::toolBox::write_log( 'Deconnexion de la base de donnees OBM', 'W' );
if( !&OBM::dbUtils::dbState( 'disconnect', \$dbHandler ) ) {
    &OBM::toolBox::write_log( 'Probleme lors de la fermeture de la base de donnees...', 'W' );
}


# On ferme le log
&OBM::toolBox::write_log( "Execution du script terminee", "WC" );

exit $errorCode;
